/**
  * Aller (le terrier d'AbulEdu)
  *
  * @warning aucun traitement d'erreur n'est pour l'instant implémenté
  * @see https://redmine.ryxeo.com/projects/leterrier-aller
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "abuledupiece.h"
#include <QDebug>
#include <abuleduscenetableau.h>
#include "abuleducellule.h"
#include "abuleduentete.h"
AbuleduPiece::AbuleduPiece(QRectF rect, QGraphicsObject *parent) :
    QGraphicsObject(parent)
{
    setZValue(10);
    setCursor(QCursor(Qt::PointingHandCursor));
    setFlag(QGraphicsItem::ItemSendsGeometryChanges);
    setFlag(QGraphicsItem::ItemIsMovable);
    m_rect = rect;
    setPen(QPen(QColor(Qt::black), 1));
    setBrush(Qt::transparent);
    m_image = QImage();
}

void AbuleduPiece::setPen(QPen contour)
{
    m_penContour = contour;
}

void AbuleduPiece::setBrush(QBrush brosseFond)
{
    m_brosseFond = brosseFond;
}

void AbuleduPiece::setPositionOriginale(QPointF pos)
{
    m_positionOriginale = pos;
    setPos(m_positionOriginale);
}

QPointF AbuleduPiece::positionOriginale()
{
    return m_positionOriginale;
}

void AbuleduPiece::setImage(QString cheminImage)
{
    m_image = QImage(cheminImage);
}

void AbuleduPiece::paint(QPainter *painter, const QStyleOptionGraphicsItem *option, QWidget *widget)
{
    // Dessin du fond et du contour
    painter->setPen(m_penContour); // Le crayon de contour
    painter->setBrush(m_brosseFond); // La couleur du fond
    painter->drawRoundedRect(m_rect, 10, 10);
    // Dessin d'une image au dessus du fond  mise à l'échelle et proportionnelle
    if(!m_image.isNull()){
        QImage image = m_image.scaled(m_rect.width()-6, m_rect.height()-6, Qt::KeepAspectRatio, Qt::SmoothTransformation); // Possibilité d'ajouter un paramètre de marge pour remplacer le -6, il faudrait aussi tenir compte de la demi épaisseur du Pen !!!
        int x = (m_rect.width() - image.width())/2;
        int y = (m_rect.height() - image.height())/2;
        painter->drawImage(x, y, image);
    }

    // Dessin du texte il faut le faire tenir dans le rectangle !!!
}

QRectF AbuleduPiece::boundingRect() const
{
    return QRectF(m_rect);
}

void AbuleduPiece::mousePressEvent(QGraphicsSceneMouseEvent *event)
{
    qDebug()<<"Je suis :"<<objectName()<<" et on m'appuie dessus, la position de la souris est :"<<event->scenePos();
    setZValue(11); // pour être au-dessus des autres
    m_hotspot = event->scenePos() - scenePos();
    qDebug()<<"la piece est a la position : "<<scenePos()<<pos()<<"hotspot"<<m_hotspot;
    event->accept();
}
//void AbuleduPiece::mouseDoubleClickEvent(QGraphicsSceneMouseEvent *event)
//{
//   event->ignore();
//}
void AbuleduPiece::mouseMoveEvent(QGraphicsSceneMouseEvent *event)
{
   event->ignore();
   setPos(event->scenePos() - m_hotspot);
}
void AbuleduPiece::mouseReleaseEvent(QGraphicsSceneMouseEvent *event)
{
    QList<QGraphicsItem *>listeItems = static_cast<QGraphicsScene *>(this->parent())->items(event->scenePos());
    listeItems.removeAt(listeItems.indexOf(this)); // On enleve cette piece de la liste
    if(listeItems.size() == 0){
        setPos(positionOriginale());
    }

    if(listeItems.size() == 1){
        if(listeItems[0]->type() == AbuleduCellule::AbuleduCelluleType){ //On est sur une cellule vide, on place la piece au mileu
            qDebug() << "Type : " << listeItems[0]->type();
            setPos(listeItems[0]->x()+(listeItems[0]->boundingRect().width() - this->boundingRect().width())/2,
                   listeItems[0]->y()+(listeItems[0]->boundingRect().height()-this->boundingRect().height())/2);
            emit pieceDeposee(objectName(), static_cast<AbuleduCellule *>(listeItems[0])->objectName().toInt());
        }
        // NSE
        if(listeItems[0]->type() == AbuleduEntete::AbuleduEnteteType){ //On est sur une entete, on revient a la position initiale
            qDebug() << "RETOUR Position initiale - Type : " << listeItems[0]->type();
            setPos(positionOriginale());
        }
    }

    // Todo cas ou il y a déjà une piece dans la cellule

    // Todo cas ou on essaie de la déposer sur une autre piece

    setZValue(10); // On la remet à son niveau initial
}
QVariant AbuleduPiece::itemChange(GraphicsItemChange change, const QVariant &value)
 {
     if (change == ItemPositionChange) {
         // value est la nouvelle position.
         QPointF newPos = value.toPointF();

         float posX = newPos.x();
         // newPos.setX(qMin((float)450,posX)); // Butée à droite pour voir
         // NSE : new version
         newPos.setX(qMin((float)600,posX)); // Butée à droite pour voir

             return newPos; // newPOs est bien un QVariant
         }

     return QGraphicsItem::itemChange(change, value);
 }
