/**
  * Aller (le terrier d'AbulEdu)
  *
  * @warning aucun traitement d'erreur n'est pour l'instant implémenté
  * @see https://redmine.ryxeo.com/projects/leterrier-aller
  * @author 2011 Jean-Louis Frucot <frucot.jeanlouis@free.fr>
  * @see The GNU Public License (GPL)
  */

/*
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 * or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details.
 *
 * You should have received a copy of the GNU General Public License along
 * with this program; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */

#include "abuledutableau.h"
#include <QDebug>

AbuleduTableau::AbuleduTableau(QPointF position, QWidget *parent) :
    QGraphicsView(parent)
{
    m_position = position;
    m_scene = new AbuleduSceneTableau(this);
    m_scene->setObjectName("La QGraphicsScene");
    setScene(m_scene);
}

QMap<int, AbuleduCellule *> AbuleduTableau::dessineTableau(QPointF position, int nbColonnes, int largeurColonne, int nbLignes, int hauteurLigne)
{
    QMap<int, AbuleduCellule *> listeCellules;
    listeCellules.clear();
    for(int j=0; j<nbLignes; j++){
        int i=0;
        for(i; i<nbColonnes; i++){
            AbuleduCellule *cellule = new AbuleduCellule(QSizeF(largeurColonne, hauteurLigne));
            m_scene->addItem(cellule);
            cellule->setPos(i*largeurColonne + position.x(), j*hauteurLigne + position.y());
            cellule->setObjectName(QString::number(j*nbColonnes+i)); //numérotage des cellules
            // NSE
            cellule->setBrush(Qt::white);
            listeCellules.insert(j*nbColonnes+i, cellule);
        }
    }
    return listeCellules;
}

// NSE
QMap<int, AbuleduEntete *> AbuleduTableau::dessineEnteteTableau(QPointF position, int nbColonnes, int largeurColonne, int nbLignes, int hauteurLigne)
{
    QMap<int, AbuleduEntete *> listeEntetes;
    listeEntetes.clear();

    QMap<int, AbuleduEntete *> listeEntetesLigne;
    listeEntetesLigne.clear();
    QMap<int, AbuleduEntete *> listeEntetesCol;
    listeEntetesCol.clear();

    // ajoute entetes de lignes
    for(int j=0; j<nbLignes; j++){
        AbuleduEntete *entete = new AbuleduEntete(QSizeF(largeurColonne, hauteurLigne));
        m_scene->addItem(entete);
        entete->setPos(position.x() - largeurColonne, j*hauteurLigne + position.y());
        entete->setObjectName(QString::number(j)); //numérotage des cellules
        entete->setBrush(Qt::blue);
        listeEntetes.insert(j, entete);

        // Mémorise les entetes de Lignes
        listeEntetesLigne.insert(j, entete);
    }

    // ajoute entetes de colonnes
    for(int i = 0; i<nbColonnes; i++){
            AbuleduEntete *entete = new AbuleduEntete(QSizeF(largeurColonne, hauteurLigne));
            m_scene->addItem(entete);
            entete->setPos(i*largeurColonne + position.x(), position.y() - hauteurLigne);
            entete->setBrush(Qt::red);
            entete->setObjectName(QString::number(nbLignes+i)); //numérotage des cellules
            listeEntetes.insert(nbLignes+i, entete);

            // Mémorise les entetes de Lignes
            listeEntetesCol.insert(i, entete);
    }
    return listeEntetes;
}


// NSE
QMap<int, AbuleduEntete *> AbuleduTableau::dessineEnteteTableauLigne(QPointF position, int nbColonnes, int largeurColonne, int nbLignes, int hauteurLigne, QString posLigne)
{
    QMap<int, AbuleduEntete *> listeEntetesLigne;

    listeEntetesLigne.clear();

    // ajoute entetes de lignes
    for(int j=0; j<nbLignes; j++){
        AbuleduEntete *entete = new AbuleduEntete(QSizeF(largeurColonne, hauteurLigne));
        m_scene->addItem(entete);
        if (posLigne == "G") {
            // Positionnement des Entetes de Lignes à Gauche
            entete->setPos(position.x() - largeurColonne, j*hauteurLigne + position.y());
        }
        if (posLigne == "D") {
            // Positionnement des Entetes de Lignes à Droite
            entete->setPos(position.x() + nbColonnes*largeurColonne, j*hauteurLigne + position.y());
        }
        entete->setObjectName(QString::number(j)); //numérotage des cellules
        entete->setBrush(Qt::blue);

        // Mémorise les entetes de Lignes
        listeEntetesLigne.insert(j, entete);
    }
    return listeEntetesLigne;
}

// NSE
QMap<int, AbuleduEntete *> AbuleduTableau::dessineEnteteTableauColonne(QPointF position, int nbColonnes, int largeurColonne, int nbLignes, int hauteurLigne, QString posColonne)
{
    QMap<int, AbuleduEntete *> listeEntetesColonne;

    listeEntetesColonne.clear();

    // ajoute entetes de colonnes
    for(int i=0; i<nbColonnes; i++){
        AbuleduEntete *entete = new AbuleduEntete(QSizeF(largeurColonne, hauteurLigne));
        m_scene->addItem(entete);
        if (posColonne == "H") {
            // Positionnement des Entetes de Colonnes en Haut
            entete->setPos(i*largeurColonne + position.x(), position.y() - hauteurLigne);
        }
        if (posColonne == "B") {
            // Positionnement des Entetes de Colonnes en Bas
            entete->setPos(i*largeurColonne + position.x(), position.y() + nbLignes*hauteurLigne);
        }
        entete->setObjectName(QString::number(i)); //numérotage des cellules
        entete->setBrush(Qt::red);

        // Mémorise les entetes de Lignes
        listeEntetesColonne.insert(i, entete);
    }
    return listeEntetesColonne;
}



void AbuleduTableau::setDimensionsWidget(QSize taille)
{
    setFixedSize(taille);
    setSceneRect(0,0,taille.width(), taille.height());
    setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
    setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
}
